package main

import (
	"crypto/hmac"
	"crypto/sha256"
	"encoding/base64"
	"fmt"
	"net/url"
	"strconv"
	"strings"
	"time"
)

const (
	contentTypePrefix              = "application/json"
	signatureHeadersWithoutContent = "(request-target) host date"
	signatureHeadersWithContent    = signatureHeadersWithoutContent + " content-type content-length"
)

// GenerateAuthHeaders generates a map with required authorization headers (Date, Authorization).
//
// apiKey     user's API key.
// apiSecret  user's API secret.
// httpMethod HTTP method of a request (e.g. "get", "options", "head", "post", "put", "patch", or "delete").
// rawURL     URL to send the request to (e.g. "https://www.example.com/resources").
//
// returns a map with all required headers for authorization (Date, Authorization)
// or an error.
func GenerateAuthHeaders(
	apiKey string,
	apiSecret string,
	httpMethod string,
	rawURL string) (map[string]string, error) {

	return generateAuthHeaders(apiKey, apiSecret, httpMethod, rawURL, nil, nil)
}

// GenerateAuthHeadersWithContent generates a map with required authorization headers
// (Date, Authorization, Content-Type, Content-Length).
//
// apiKey      user's API key.
// apiSecret   user's API secret.
// httpMethod  HTTP method of a request (e.g. "get", "options", "head", "post", "put", "patch", or "delete").
// rawURL      URL to send the request to (e.g. "https://www.example.com/resources").
// contentType content type of the request payload/body (e.g. "application/json").
// payload     the request payload/body
//
// returns a map with all required headers for authorization (Date, Authorization, Content-Type, Content-Length)
// or an error.
func GenerateAuthHeadersWithContent(
	apiKey string,
	apiSecret string,
	httpMethod string,
	rawURL string,
	contentType string,
	payload string) (map[string]string, error) {

	return generateAuthHeaders(apiKey, apiSecret, httpMethod, rawURL, &contentType, &payload)
}

func generateAuthHeaders(
	apiKey string,
	apiSecret string,
	httpMethod string,
	rawURL string,
	contentType *string,
	payload *string) (map[string]string, error) {

	err := validatePayload(contentType, payload)
	if err != nil {
		return nil, err
	}

	date, err := generateDateHeader()
	if err != nil {
		return nil, err
	}

	authorization, err := generateAuthorizationHeader(apiKey, apiSecret, httpMethod, rawURL, contentType, payload, date)
	if err != nil {
		return nil, err
	}

	authHeaders := map[string]string{
		"Date":          date,
		"Authorization": authorization,
	}

	if payload != nil {
		authHeaders["Content-Type"] = *contentType
		authHeaders["Content-Length"] = strconv.Itoa(len(*payload))
	}

	return authHeaders, nil
}

func validatePayload(
	contentType *string,
	payload *string) error {

	var err error = nil

	if contentType != nil && !strings.HasPrefix(*contentType, contentTypePrefix) {
		err = fmt.Errorf("unsupported content type %s", *contentType)
	} else if contentType != nil && payload == nil {
		err = fmt.Errorf("the request payload(body) has not been provided")
	} else if contentType == nil && payload != nil {
		err = fmt.Errorf("the content type of request payload(body) has not been provided")
	}

	return err
}

// generateDateHeader returns current date and time in GMT timezone or an error.
// It uses RFC 1123 date format (e.g. "Wed, 25 May 2022 12:53:15 GMT").
func generateDateHeader() (string, error) {
	location, err := time.LoadLocation("GMT")
	if err != nil {
		return "", err
	}

	return time.Now().In(location).Format(time.RFC1123), nil
}

func generateAuthorizationHeader(
	apiKey string,
	apiSecret string,
	httpMethod string,
	rawURL string,
	contentType *string,
	payload *string,
	date string) (string, error) {

	headers := getHeaders(payload)
	signature, err := generateSignature(apiSecret, httpMethod, rawURL, contentType, payload, date)
	if err != nil {
		return "", err
	}

	authHeader := `Signature ` +
		`keyId="` + apiKey + `",` +
		`algorithm="hmac-sha256",` +
		`headers="` + headers + `",` +
		`signature="` + signature + `"`

	return authHeader, nil
}

func getHeaders(payload *string) string {
	if payload == nil {
		return signatureHeadersWithoutContent
	}

	return signatureHeadersWithContent
}

func generateSignature(
	apiSecret string,
	httpMethod string,
	rawURL string,
	contentType *string,
	payload *string,
	date string) (string, error) {

	parsedURL, err := url.Parse(rawURL)
	if err != nil {
		return "", err
	}

	dataToSign := "(request-target): " + strings.ToLower(httpMethod) + " " + parsedURL.EscapedPath() + "\n" +
		"host: " + parsedURL.Host + "\n" +
		"date: " + date

	if payload != nil {
		dataToSign += "\n" +
			"content-type: " + *contentType + "\n" +
			"content-length: " + strconv.Itoa(len(*payload)) + "\n" +
			*payload
	}

	return signDataWithHmacSha256(apiSecret, dataToSign), nil
}

func signDataWithHmacSha256(
	apiSecret string,
	dataToSign string) string {

	hashDigest := hmac.New(sha256.New, []byte(apiSecret))
	hashDigest.Write([]byte(dataToSign))
	return base64.StdEncoding.EncodeToString(hashDigest.Sum(nil))
}
