Java
====

This file contains prerequisites and examples for working with **AuthHeadersBuilder** library using `Unirest`, 
`Asynchttp` or `OkHttp` lib.

Unirest
----

### Prerequisites:

Request examples below are based on the external `Unirest` library. To use it add corresponding dependencies to
the `pom.xml` file:

    <dependency>
        <groupId>com.mashape.unirest</groupId>
        <artifactId>unirest-java</artifactId>
        <version>${unirest.version}</version>
    </dependency>

Then import the needed modules:

    import com.mashape.unirest.http.HttpResponse;
    import com.mashape.unirest.http.Unirest;
    import com.mashape.unirest.http.exceptions.UnirestException;

### Examples:

- GET request:

        protected static void sendGetRequest() throws URISyntaxException, UnirestException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "get", new URI(uri));

            HttpResponse<String> response =
                Unirest.get(uri)
                       .headers(authHeaders)
                       .asString();

            System.out.println("Status: " + response.getStatus());
            System.out.println("Body: " + response.getBody());
        }

- POST request:

        protected static void sendPostRequest() throws URISyntaxException, UnirestException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";
            String contentType = "application/json";
            String payload = "REQUEST_BODY";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "post", new URI(uri), contentType, payload);
            authHeaders.remove("Content-Length");

            HttpResponse<String> response =
                Unirest.post(uri)
                       .headers(authHeaders)
                       .body(payload)
                       .asString();

            System.out.println("Status: " + response.getStatus());
            System.out.println("Body: " + response.getBody());
        }

- PUT request

        protected static void sendPutRequest() throws URISyntaxException, UnirestException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";
            String contentType = "application/json";
            String payload = "REQUEST_BODY";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "put", new URI(uri), contentType, payload);
            authHeaders.remove("Content-Length");

            HttpResponse<String> response =
                Unirest.put(uri)
                       .headers(authHeaders)
                       .body(payload)
                       .asString();

            System.out.println("Status: " + response.getStatus());
            System.out.println("Body: " + response.getBody());
        }

- DELETE request

        protected static void sendDeleteRequest() throws URISyntaxException, UnirestException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "delete", new URI(uri));

            HttpResponse<String> response =
                Unirest.delete(uri)
                       .headers(authHeaders)
                       .asString();

            System.out.println("Status: " + response.getStatus());
            System.out.println("Body: " + response.getBody());
        }

Asynchttp
----

### Prerequisites:

Request examples below are based on the external `Asynchttp` library. To use it add corresponding dependencies to
the `pom.xml` file:

    <dependency>
        <groupId>org.asynchttpclient</groupId>
        <artifactId>async-http-client</artifactId>
        <version>${asynchttp.version}</version>
    </dependency>

Then import the needed modules:

    import org.asynchttpclient.AsyncHttpClient;
    import org.asynchttpclient.DefaultAsyncHttpClient;

### Examples:

- GET request:

        protected static void sendGetRequest() throws URISyntaxException, IOException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "get", new URI(uri));

            AsyncHttpClient client = new DefaultAsyncHttpClient();

            client.prepareGet(uri)
                  .setSingleHeaders(authHeaders)
                  .execute()
                  .toCompletableFuture()
                  .thenAccept(System.out::println)
                  .join();

            client.close();
        }

- POST request:

        protected static void sendPostRequest() throws URISyntaxException, IOException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";
            String contentType = "application/json";
            String payload = "REQUEST_BODY";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "post", new URI(uri), contentType, payload);

            AsyncHttpClient client = new DefaultAsyncHttpClient();

            client.preparePost(uri)
                  .setSingleHeaders(authHeaders)
                  .setBody(payload)
                  .execute()
                  .toCompletableFuture()
                  .thenAccept(System.out::println)
                  .join();

            client.close();
        }

- PUT request

        protected static void sendPutRequest() throws URISyntaxException, IOException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";
            String contentType = "application/json";
            String payload = "REQUEST_BODY";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "put", new URI(uri), contentType, payload);

            AsyncHttpClient client = new DefaultAsyncHttpClient();

            client.preparePut(uri)
                  .setSingleHeaders(authHeaders)
                  .setBody(payload)
                  .execute()
                  .toCompletableFuture()
                  .thenAccept(System.out::println)
                  .join();

            client.close();
        }

- DELETE request

        protected static void sendDeleteRequest() throws URISyntaxException, IOException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "delete", new URI(uri));

            AsyncHttpClient client = new DefaultAsyncHttpClient();

            client.prepareDelete(uri)
                  .setSingleHeaders(authHeaders)
                  .execute()
                  .toCompletableFuture()
                  .thenAccept(System.out::println)
                  .join();

            client.close();
        }

OkHttp
----

### Prerequisites:

Request examples below are based on the external `OkHttp` library. To use it add corresponding dependencies to
the `pom.xml` file:

    <dependency>
        <groupId>com.squareup.okhttp3</groupId>
        <artifactId>okhttp</artifactId>
        <version>${okhttp.version}</version>
    </dependency>

Then import the needed modules:

    import okhttp3.*;

### Examples:

- GET request:

        protected static void sendGetRequest() throws IOException, URISyntaxException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "get", new URI(uri));

            OkHttpClient client = new OkHttpClient();

            Request request = new Request.Builder().url(uri)
                                                   .get()
                                                   .headers(Headers.of(authHeaders))
                                                   .build();

            Response response = client.newCall(request).execute();

            System.out.println("Status: " + response.code());
            System.out.println("Body: " + response.body().string());
        }

- POST request:

        protected static void sendPostRequest() throws IOException, URISyntaxException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";
            String contentType = "application/json";
            String payload = "REQUEST_BODY";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "post", new URI(uri), contentType, payload);

            OkHttpClient client = new OkHttpClient();

            Request request = new Request.Builder().url(uri)
                                                   .post(RequestBody.create(payload.getBytes()))
                                                   .headers(Headers.of(authHeaders))
                                                   .build();

            Response response = client.newCall(request).execute();

            System.out.println("Status: " + response.code());
            System.out.println("Body: " + response.body().string());
        }

- PUT request

        protected static void sendPutRequest() throws IOException, URISyntaxException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";
            String contentType = "application/json";
            String payload = "REQUEST_BODY";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "put", new URI(uri), contentType, payload);

            OkHttpClient client = new OkHttpClient();

            Request request = new Request.Builder().url(uri)
                                                   .put(RequestBody.create(payload.getBytes()))
                                                   .headers(Headers.of(authHeaders))
                                                   .build();

            Response response = client.newCall(request).execute();

            System.out.println("Status: " + response.code());
            System.out.println("Body: " + response.body().string());
        }

- DELETE request

        protected static void sendDeleteRequest() throws IOException, URISyntaxException {
            String apiKey = "MY_API_KEY";
            String apiSecret = "MY_API_SECRET";
            String uri = "https://www.example.com/resources";

            Map<String, String> authHeaders = 
                AuthHeadersBuilder.generateAuthHeaders(apiKey, apiSecret, "delete", new URI(uri));

            OkHttpClient client = new OkHttpClient();

            Request request = new Request.Builder().url(uri)
                                                   .delete()
                                                   .headers(Headers.of(authHeaders))
                                                   .build();

            Response response = client.newCall(request).execute();

            System.out.println("Status: " + response.code());
            System.out.println("Body: " + response.body().string());
        }
