Node js
====

This file contains prerequisites and examples for working with **auth_headers_builder.js** library using native `https` 
lib or external libraries such as `Unirest for Node.js` and `Axios`.

Node Native (https lib)
----

### Prerequisites:

The request examples below are based on internal 'https' library.
To use this you do not need to install something new.

To import the needed modules:

    const request = require('https');
    const authHeadersBuilder = require("./auth_headers_builder");

where _'https'_ is the native lib for making requests and
_'auth_headers_builder'_ is a lib for generating authorization headers within _hmac_auth_ algorithm.

### Examples:

- GET request

      const http = require("https");
      let authHeadersBuilder = require("./auth_headers_builder");

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "GET";
      const url = "https://www.example.com/resources";
      const parsedUrl = new URL(url);

      const authHeaders = authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url);

      const options = {
          "method": method,
          "hostname": parsedUrl.host,
          "port": null,
          "path": parsedUrl.pathname,
          "headers": authHeaders
      };

      const req = http.request(options, function (res) {
          const chunks = [];

          res.on("data", function (chunk) {
              chunks.push(chunk);
          });

          res.on("end", function () {
              console.log(res.statusCode)
              const body = Buffer.concat(chunks);
              console.log(body.toString());
          });
      });
      req.end();

- POST request:

      const http = require("https");
      let authHeadersBuilder = require("./auth_headers_builder")

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "POST";
      const url = "https://www.example.com/resources";
      const parsedUrl = new URL(url);
      const contentType = "application/json";

      const payload = JSON.stringify({
          field1: 'value',
          field2: ['value'],
      });

      const authHeaders = 
          authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url, contentType, payload);

      const options = {
          "method": method,
          "hostname": parsedUrl.host,
          "port": null,
          "path": parsedUrl.pathname,
          "headers": authHeaders
      };

      const req = http.request(options, function (res) {
          const chunks = [];

          res.on("data", function (chunk) {
              chunks.push(chunk);
          });

          res.on("end", function () {
              const body = Buffer.concat(chunks);
              console.log(res.statusCode);
              console.log(body.toString());
          });
      });
      req.write(payload);
      req.end();

- PUT request

      const http = require("https");
      let authHeadersBuilder = require("./auth_headers_builder");

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "PUT";
      const url = "https://www.example.com/resources";
      const parsedUrl = new URL(url);
      const contentType = "application/json";

      const payload = JSON.stringify({field: ['value']});

      const authHeaders = 
          authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url, contentType, payload);

      const options = {
          "method": method,
          "hostname": parsedUrl.host,
          "port": null,
          "path": parsedUrl.pathname,
          "headers": authHeaders
      };

      const req = http.request(options, function (res) {
          const chunks = [];

          res.on("data", function (chunk) {
              chunks.push(chunk);
          });

          res.on("end", function () {
              console.log(res.statusCode)
              console.log(res.body);
          });
      });
      req.write(payload);
      req.end();

- DELETE request:

      const http = require("https");
      const authHeadersBuilder = require("./auth_headers_builder");

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "DELETE";
      const url = "https://www.example.com/resources";
      const parsedUrl = new URL(url);

      const authHeaders = authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url);

      const options = {
          "method": method,
          "hostname": parsedUrl.host,
          "port": null,
          "path": parsedUrl.pathname,
          "headers": authHeaders
      };

      const req = http.request(options, function (res) {
          const chunks = [];

          res.on("data", function (chunk) {
              chunks.push(chunk);
          });

          res.on("end", function () {
              console.log(res.statusCode);
              console.log(res.body);
          });
      });
      req.end();

Unirest lib
----

### Prerequisites:

The request examples below are based on external 'unirest' library.
To use them install the lib by running the following command:

     npm install unirest

Then import the needed modules:

    const unirest = require("unirest");
    const authHeadersBuilder = require("./auth_headers_builder");

where _'unirest'_ is the lib installed in the previous step for making requests and
_'auth_headers_builder'_ is a lib for generating authorization headers within _hmac_auth_ algorithm.

### Examples:

- GET request:

      const unirest = require("unirest");
      const authHeadersBuilder = require("./auth_headers_builder");

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "GET";
      const url = "https://www.example.com/resources";

      const authHeaders = authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url);

      const req = unirest(method, url);

      req.headers(authHeaders);

      req.end(function (res) {
          if (res.error) {
              throw new Error(res.error);
          }
          console.log(res.statusCode);
          console.log(res.body);
      });

- POST request:

      const unirest = require("unirest");
      const authHeadersBuilder = require("./auth_headers_builder");

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "POST";
      const url = "https://www.example.com/resources";
      const contentType = "application/json";

      let payloadRaw = {
          "field1": "value",
          "field2": "value",
      };

      let payload = JSON.stringify(payloadRaw);

      const authHeaders =
          authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url, contentType, payload);

      const req = unirest(method, url);

      req.headers(authHeaders);

      req.type("json");
      req.send(payload);
      req.end(function (res) {
          if (res.error) {
              throw new Error(res.error);
          }
          console.log(res.statusCode);
          console.log(res.body);
      });

- PUT request:

      const unirest = require("unirest");
      const authHeadersBuilder = require("./auth_headers_builder");

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "PUT";
      const url = "https://www.example.com/resources";
      const contentType = "application/json";

      let payloadRaw = {
          "field": ["value"]
      }

      let payload = JSON.stringify(payloadRaw);

      const req = unirest(method, url);

      const authHeaders =
          authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url, contentType, payload);

      req.headers(authHeaders);

      req.type("json");
      req.send(payloadRaw);
      req.end(function (res) {
      if (res.error) throw new Error(res.error);
          console.log(res.statusCode);
          console.log(res.body);
      });

- DELETE request:

      const unirest = require("unirest");
      const authHeadersBuilder = require("./auth_headers_builder");

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "DELETE";
      const url = "https://www.example.com/resources";

      const authHeaders = authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url);

      const req = unirest(method, url);

      req.headers(authHeaders);

      req.type("json");
      req.end(function (res) {
          if (res.error) {
              throw new Error(res.error);
          }
          console.log(res.statusCode);
          console.log(res.body);
      });

Axios lib
----

### Prerequisites:

The request examples below are based on external 'axios' library.
To use them install the lib by running the following command:

     npm install axios

Then import the needed modules:

    const unirest = require("axios");
    const authHeadersBuilder = require("./auth_headers_builder");

where _'axios'_ is the lib installed in the previous step for making requests and
_'auth_headers_builder'_ is a lib for generating authorization headers within _hmac_auth_ algorithm.

### Examples:

- GET request:

      const axios = require("axios");
      let authHeadersBuilder = require("./auth_headers_builder");

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "GET";
      const url = "https://www.example.com/resources";

      const authHeaders = authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url);

      const options = {
          method: method,
          url: url,
          headers: authHeaders
      };

      axios.request(options).then(function (response) {
          console.log(response.status);
          console.log(response.data);
      }).catch(function (error) {
          console.error(error);
      });

- POST request:
  
      const axios = require("axios");
      let authHeadersBuilder = require("./auth_headers_builder")

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "POST";
      const url = "https://www.example.com/resources";
      const contentType = "application/json";

      const payload = JSON.stringify({
          field1: 'value',
          field2: [
               {field3: 'value', field4: 'value'},
          ],
      });

      const authHeaders =
          authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url, contentType, payload);

      const options = {
          method: method,
          url: url,
          headers: authHeaders,
          data: payload
      };

      axios.request(options).then(function (response) {
          console.log('STATUS: ' + response.status)
          console.log(response.data);
      }).catch(function (error) {
          console.error(error);
      });

- PUT request:

      const axios = require("axios");
      const authHeadersBuilder = require("./auth_headers_builder");

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "PUT";
      const url = "https://www.example.com/resources";
      const contentType = "application/json";

      const payload = JSON.stringify({
          field: ['value']
      });

      const authHeaders =
          authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url, contentType, payload);

      const options = {
          method: method,
          url: url,
          headers: authHeaders,
          data: payload
      };

      axios.request(options).then(function (response) {
          console.log('STATUS: ' + response.status)
          console.log(response.data);
      }).catch(function (error) {
          console.error(error);
      });

- DELETE request:

      const axios = require("axios");
      let authHeadersBuilder = require("./auth_headers_builder")

      const apiKey = "MY_API_KEY";
      const apiSecret = "MY_API_SECRET";
      const method = "DELETE";
      const url = "https://www.example.com/resources";

      const authHeaders = authHeadersBuilder.generateAuthorizationHeaders(apiKey, apiSecret, method, url);

      const options = {
          method: method,
          url: url,
          headers: authHeaders,
          data: null
      };

      axios.request(options).then(function (response) {
          console.log('STATUS: ' + response.status)
          console.log(response.data);
      }).catch(function (error) {
          console.error(error);
      });
