<?php

class AuthHeadersBuilder {

    private const CONTENT_TYPE_PREFIX = 'application/json';

    private const SIGNATURE_HEADERS = '(request-target) host date';

    private const SIGNATURE_HEADERS_WITH_CONTENT = self::SIGNATURE_HEADERS . ' content-type content-length';

    /**
     * Generates an associative array with required authorization headers (Date, Authorization, Content-Type, Content-Length).
     *
     * @param string $apiKey User's API key.
     * @param string $apiSecret User's API secret.
     * @param string $httpMethod HTTP method of a request (e.g. "get", "options", "head", "post", "put", "patch", or "delete").
     * @param string $url URL to send the request to (e.g. "https://www.example.com/resources").
     * @param string|null $contentType (optional) Content type of the request payload/body (e.g. "application/json").
     * @param string|null $payload (optional) The request payload/body.
     * @return array (an associative array) with required headers for authorization
     * (Date, Authorization, Content-Type, Content-Length).
     * @throws InvalidArgumentException if there is only one $contentType or $payload value, or if $contentType is not supported.
     * Currently, only the $contentType value "application/json" is supported.
     */
    public static function generateAuthHeaders(string $apiKey,
                                               string $apiSecret,
                                               string $httpMethod,
                                               string $url,
                                               string $contentType = null,
                                               string $payload = null): array {
        self::validatePayload($contentType, $payload);

        $date = self::generateDateHeader();
        $authorization = self::generateAuthorizationHeader($apiKey, $apiSecret, $httpMethod, $url, $contentType, $payload, $date);

        $authHeaders = [
            'Date' => $date,
            'Authorization' => $authorization
        ];

        if ($payload != null) {
            $authHeaders['Content-Type'] = $contentType;
            $authHeaders['Content-Length'] = strval(strlen($payload));
        }

        return $authHeaders;
    }

    private static function validatePayload($contentType, $payload): void {
        if ($contentType != null && strpos($contentType, self::CONTENT_TYPE_PREFIX) !== 0) {
            throw new InvalidArgumentException("Unsupported content type $contentType");
        }

        if ($contentType != null && $payload == null) {
            throw new InvalidArgumentException('The request payload(body) has not been provided');
        }

        if ($contentType == null && $payload != null) {
            throw new InvalidArgumentException('The content type of request payload(body) has not been provided');
        }
    }

    /**
     * Generates Date header value.
     *
     * @return string Current date and time in GMT timezone in RFC 1123 date format (e.g. "Wed, 25 May 2022 12:53:15 GMT").
     */
    private static function generateDateHeader(): string {
        return gmdate('D, d M Y H:i:s \G\M\T', time());
    }

    private static function generateAuthorizationHeader(string $apiKey,
                                                        string $apiSecret,
                                                        string $httpMethod,
                                                        string $url,
                                                        ?string $contentType,
                                                        ?string $payload,
                                                        string $date): string {
        $headers = $payload == null ? self::SIGNATURE_HEADERS : self::SIGNATURE_HEADERS_WITH_CONTENT;
        $signature = self::generateSignature($apiSecret, $httpMethod, $url, $contentType, $payload, $date);

        return "Signature keyId=\"$apiKey\",algorithm=\"hmac-sha256\",headers=\"$headers\",signature=\"$signature\"";
    }

    private static function generateSignature(string $apiSecret,
                                              string $httpMethod,
                                              string $url,
                                              ?string $contentType,
                                              ?string $payload,
                                              string $date): string {
        $path = parse_url($url, PHP_URL_PATH);
        $host = parse_url($url, PHP_URL_HOST);
        $method = strtolower($httpMethod);

        $dataToSign = "(request-target): $method $path\nhost: $host\ndate: $date";

        if ($payload != null) {
            $contentLength = strlen($payload);
            $dataToSign .= "\ncontent-type: $contentType\ncontent-length: $contentLength\n$payload";
        }

        return self::signDataWithHmacSha256($apiSecret, $dataToSign);
    }

    private static function signDataWithHmacSha256(string $apiSecret, string $dataToSign): string {
        $hashDigest = hash_init("sha256", HASH_HMAC, $apiSecret);
        hash_update($hashDigest, $dataToSign);

        return base64_encode(hash_final($hashDigest, TRUE));
    }
}
