Python
====

This file contains prerequisites and examples for working with auth_headers_builder library using requests lib or
default python3 module http.client.

Requests lib
----

### Prerequisites:

The request examples below are based on external 'requests' library.
To use them install the lib by running the following command:

     pip install requests

Then import the needed modules:

    import requests
    from auth_headers_builder import generate_auth_headers

where _requests_ is the lib installed in the previous step for making requests,
_generate_auth_headers_ is a method for generating authorization headers in _auth_headers_builder_ library.

### Examples:

- GET request:

        import requests
        from auth_headers_builder import generate_auth_headers
        
        if __name__ == '__main__':
            api_key = 'MY_API_KEY'
            api_secret = 'MY_API_SECRET'
            url = 'https://www.example.com/resources'
        
            auth_headers = generate_auth_headers(api_key, api_secret, 'get', url)
        
            headers = {
                'Date': auth_headers.date,
                'Authorization': auth_headers.authorization
            }
        
            response = requests.request('GET', url, headers=headers)
        
            print('status: ', response.status_code)
            print('body: ', response.text)

- POST request:

        import requests
        from auth_headers_builder import generate_auth_headers
    
        if __name__ == '__main__':
            api_key = 'MY_API_KEY'
            api_secret = 'MY_API_SECRET'
            url = 'https://www.example.com/resources'
            content_type = 'application/json'
            payload = {
                'groupId': 'GROUP_ID',
                'entityType': 'INDIVIDUAL',
                'name': 'NAME',
                'providerTypes': [
                    'WATCHLIST',
                    'MEDIA_CHECK'
                ],
                'secondaryFields': [],
                'customFields': []
            }
            
            auth_headers = generate_auth_headers(api_key, api_secret, 'post', url, content_type, payload)
            
            headers = {
                'Date': auth_headers.date,
                'Authorization': auth_headers.authorization,
                'Content-Type': auth_headers.content_type,
                'Content-Length': auth_headers.content_length,
            }
            
            response = requests.request('POST', url, json=payload, headers=headers)
        
            print('status: ', response.status_code)
            print('body: ', response.text)

- PUT request

        import requests
        from auth_headers_builder import generate_auth_headers
        
        if __name__ == '__main__':
            api_key = 'MY_API_KEY'
            api_secret = 'MY_API_SECRET'
            url = 'https://www.example.com/resources'
            content_type = 'application/json'
            payload = {
                'providerTypes': [
                    'WATCHLIST',
                    'MEDIA_CHECK'
                ]
            }
        
            auth_headers = generate_auth_headers(api_key, api_secret, 'put', url, content_type, payload)
        
            headers = {
                'Date': auth_headers.date,
                'Authorization': auth_headers.authorization,
                'Content-Type': auth_headers.content_type,
                'Content-Length': auth_headers.content_length,
            }
        
            response = requests.request('PUT', url, json=payload, headers=headers)
        
            print('status: ', response.status_code)
            print('body: ', response.text)

- DELETE request

        import requests
        from auth_headers_builder import generate_auth_headers
        
        if __name__ == '__main__':
            api_key = 'MY_API_KEY'
            api_secret = 'MY_API_SECRET'
            url = 'https://www.example.com/resources'
        
            auth_headers = generate_auth_headers(api_key, api_secret, 'delete', url)
        
            headers = {
                'Date': auth_headers.date,
                'Authorization': auth_headers.authorization
            }
        
            response = requests.request('DELETE', url, headers=headers)
        
            print('status: ', response.status_code)
            print('body: ', response.text)

Python3
----

### Prerequisites:

Request examples below are based on python3 module http.client.
To use them install Python version 3 and import the needed modules as following:

    import http.client
    import json
    from auth_headers_builder import generate_auth_headers

where _http.client_ is the internal module that defines classes which implement the client side of the HTTP and HTTPS
protocols,
_json_ is a module for converting and working with JSON format,
_generate_auth_headers_ is a method for generating authorization headers in _auth_headers_builder_ library.

### Examples:

- GET request:

        import http.client
        from auth_headers_builder import generate_auth_headers
        
        if __name__ == '__main__':
            api_key = 'MY_API_KEY'
            api_secret = 'MY_API_SECRET'
            url = 'https://www.example.com/resources'
        
            auth_headers = generate_auth_headers(api_key, api_secret, 'get', url)

            headers = {
                'Date': auth_headers.date,
                'Authorization': auth_headers.authorization
            }
        
            conn = http.client.HTTPSConnection('www.example.com')
            conn.request('GET', '/resources', None, headers)
        
            res = conn.getresponse()
            data = res.read()
        
            print(data.decode('utf-8'))
            print(res.getcode())

- POST request:

        import http.client
        import json
        from auth_headers_builder import generate_auth_headers
        
        if __name__ == '__main__':
            api_key = 'MY_API_KEY'
            api_secret = 'MY_API_SECRET'
            url = 'https://www.example.com/resources'
            content_type = 'application/json'
            payload = {
                'groupId': 'GROUP_ID',
                'entityType': 'INDIVIDUAL',
                'name': 'NAME',
                'providerTypes': [
                    'WATCHLIST',
                    'MEDIA_CHECK'
                ],
                'secondaryFields': [],
                'customFields': []
            }
        
            auth_headers = generate_auth_headers(api_key, api_secret, 'post', url, content_type, payload)

            headers = {
                'Date': auth_headers.date,
                'Authorization': auth_headers.authorization,
                'Content-Type': auth_headers.content_type,
                'Content-Length': auth_headers.content_length
            }
        
            conn = http.client.HTTPSConnection('www.example.com')
            conn.request('POST', '/resources', json.dumps(payload), headers)
        
            res = conn.getresponse()
            data = res.read()
        
            print(data.decode('utf-8'))
            print(res.getcode())

- PUT request

      import http.client
      import json
      from auth_headers_builder import generate_auth_headers

      if __name__ == '__main__':
          api_key = 'MY_API_KEY'
          api_secret = 'MY_API_SECRET'
          url = 'https://www.example.com/resources'
          content_type = 'application/json'
          payload = {
              'providerTypes': [
                  'WATCHLIST',
                  'MEDIA_CHECK'
              ]
          }

          auth_headers = generate_auth_headers(api_key, api_secret, 'put', url, content_type, payload)

          headers = {
              'Date': auth_headers.date,
              'Authorization': auth_headers.authorization,
              'Content-Type': auth_headers.content_type,
              'Content-Length': auth_headers.content_length
          }

          conn = http.client.HTTPSConnection('www.example.com')
          conn.request('PUT', '/resources', json.dumps(payload), headers)

          res = conn.getresponse()
          data = res.read()

          print(data.decode('utf-8'))
          print(res.getcode())


- DELETE request

       import http.client
       from auth_headers_builder import generate_auth_headers
        
       if __name__ == '__main__':
           api_key = 'MY_API_KEY'
           api_secret = 'MY_API_SECRET'
           url = 'https://www.example.com/resources'
        
           auth_headers = generate_auth_headers(api_key, api_secret, 'delete', url)

           headers = {
               'Date': auth_headers.date,
               'Authorization': auth_headers.authorization
           }
        
           conn = http.client.HTTPSConnection('www.example.com')
           conn.request('DELETE', '/resources', None, headers)
        
           res = conn.getresponse()
           data = res.read()
        
           print(data.decode('utf-8'))
           print(res.getcode())
