import hmac
import hashlib
import base64
import time
import json
from urllib.parse import urlparse


def generate_auth_headers(api_key: str,
                          api_secret: str,
                          method: str,
                          url: str,
                          content_type: str = None,
                          payload: dict = None):
    """Generates object that contains authorization headers (Date, Authorization, Content-Type, Content-Length).

    :param api_key: User's API key.
    :param api_secret: User's API secret.
    :param method: HTTP method of a request (e.g. 'get', 'options', 'head', 'post', 'put', 'patch', or 'delete').
    :param url: Full form URL (e.g. 'https://www.example.com/resources').
    :param content_type: (optional) Content type header value (e.g. 'application/json'). If value is not present it is set to None.
    :param payload: (optional) Dictionary object representing content. If value is not present it is set to None.
    :returns object containing Date, Authorization, Content-Type, Content-Length headers values
    :raises ValueError if there is only one content_type or payload value, or if content_type is not supported. Currently, the script only supports the content_type value 'application / json'.

    See also README.md file for examples.
    """
    validate_payload(content_type, payload)

    auth_headers = type('', (), {})()
    date = time.strftime('%a, %d %b %Y %H:%M:%S GMT', time.gmtime())
    parsed_url = urlparse(url)
    data_to_sign = '(request-target): ' + method.lower() + ' ' + parsed_url.path + '\n' \
                   + 'host: ' + str(parsed_url.hostname) + '\n' \
                   + 'date: ' + date

    if payload is not None:
        content = json.dumps(payload)
        content_length = str(len(content))
        auth_headers.content_type = content_type
        auth_headers.content_length = content_length
        data_to_sign += '\ncontent-type: ' + content_type + '\n' \
                        + 'content-length: ' + content_length + '\n' \
                        + content

    hmac_base = generate_hmac_base(api_secret.encode(), data_to_sign.encode())
    headers = '(request-target) host date' if payload is None else '(request-target) host date content-type content-length'

    authorization = 'Signature keyId=\"' + api_key + '\"' \
                    + ',algorithm=\"hmac-sha256\",headers=\"' + headers + '\"' \
                    + ',signature=\"' + hmac_base + '\"'

    auth_headers.date = date
    auth_headers.authorization = authorization

    return auth_headers


def validate_payload(content_type: str, payload: dict):
    if content_type is not None and not content_type.startswith('application/json'):
        raise ValueError('Unsupported content type ' + content_type)

    if content_type is not None and payload is None:
        raise ValueError('The request payload(body) has not been provided')

    if content_type is None and payload is not None:
        raise ValueError('The content type of request payload(body) has not been provided')


def generate_hmac_base(api_token: bytes, byte_data_to_sign: bytes):
    encrypt = hmac.new(api_token, byte_data_to_sign, digestmod=hashlib.sha256)
    digest_maker = encrypt.digest()

    base = base64.b64encode(digest_maker)

    return base.decode()
